========================================================================================== 
Documentation for SkinMesh Support Code
EvilHomer , September 17, 2003
Chapter One - An overview of the structures and code required
========================================================================================== 

The SkinMesh Structure encapsulates eveything required for animated skinmesh models.
Our SkinMesh Structure also provides for storing multiple SkinMeshes, via its LO field.
The structure looks like this:

SkinMesh STRUCT
    LO LinkedObject <>                                       ;LinkedList support for this object
    pFramesRoot       LPFRAMENODE ?             ;Root of entire frame hierarchy of this SkinMesh Model
    pAnimationsRoot LPANIMATIONNODE ?       ;Root of entire animation set of this SkinMesh Model
SkinMesh ENDS
LPSKINMESH typedef ptr SkinMesh                 ;define LPSKINMESH data type as being a pointer to a SkinMesh

An INSTANCE of the structure is called an Object. 
An Object in our terms of reference is just a little wad of allocated memory to hold one instance of the structure.

Aside from the LO field (which contains support for naming the instance and linking it in LinkedLists),
the SkinMesh structure also contains two LinkedList root pointers which are simply dwords.
They each point to the first object in two LinkedLists owned by an instance of a SkinMesh.
The first points to the Frames hierarchy for the SkinMesh instance,
while the second points to the Animation hierarchy for the SkinMesh instance.
These two hierarchies are loaded when the instance of the SkinMesh is first created
using the "SkinMesh_Create" procedure, which is the entrypoint to our Loader procedures.

The SkinMesh_Create procedure takes just one parameter, a string pointer (filename of the xfile to be loaded).
Likewise it returns just one parameter , an LPSKINMESH pointer to the freshly loaded SkinMesh object.
A return value of E_FAIL indicates failure during loading.

The duties of the SkinMesh_Create procedure could be summarized as follows:
1- create a new instance of a SkinMesh to hold all the goodies
2- attempt to load the hierarchy from the named xfile
3- process the bone information of the skinmesh

Step #2 is made more complex due to the nature of Frame Hierarchies within XFiles.
SkinMesh_Create contains the "Top-Level Object Parser" part of our XFile Hierarchy Loader.
Basically it attempts to load the xfile hierarchy using a call to D3DXLoadSkinMeshFromXof,
then process the information returned from that call.
Processing that information includes parsing the "top-level objects" in the xfile.
We simply enumerate all of them , ie, for each one we find, we hand it to another procedure to process,
and we do that in a loop until we run out of these so-called "top-level objects".

The procedure used to process each "top-level object" we discover is called "FrameNode_Load" .
The duties of this procedure begin with identifying exactly what kind of "top-level object" we have been handed.
Having determined what it is, there's separate handlers for each possible type we might encounter.
Some of the types are relatively easy to handle, but a couple of them are a bit more complicated.
Of these, the FRAME type is the most complicated due to the fact that it can be iterated ad nauseum,
ie Frames can contain Frames that contain Frames...
This is handled in a tidy fashion using self-calling procedures aka reentrant code or nesting.
The Frames Handler code effectively calls itself from within itself, returning to itself etc.
The MESH type is also pretty heavy because theres a separate loader mechanism for loading Meshes.
The Mesh type handler code looks after that of course.

Frames are loaded into FrameNodes, and Meshes are loaded into MeshNodes.
SkinMeshes contain two root pointers to a LinkedList of each of these object types.

Here's the FrameNode Structure:

FrameNode STRUCT
	LO LinkedObject <>
	pMeshes LPMESHNODE ?		            ;	< Meshes in this Frame
	OriginalMatrix             D3DXMATRIX <>	;	< The matrix originally loaded with the frame
	TransformationMatrix D3DXMATRIX <>	;	< The transformation of the frame
	CombinedMatrix         D3DXMATRIX <>       	;            < The combined matrix
FrameNode ENDS

As we can see, the FrameNode contains LinkedList support used to keep track of frame-to-frame links.
We also see a pMeshes field, which is a root pointer to a linkedlist of Meshes owned by this Frame.
That is to say, a Frame may contain more than one Mesh !!
Furthermore, we can see it holds three matrices.
The OriginalMatrix is the one loaded from the XFile.
The TransformationMatrix is the current transformation of the Frame (combined rotation, translation, scale)
The CombinedMatrix is the one used in rending, its the combination of this frame's TransformMatrix
with its parent frame's matrix, meaning that transformations applied to a frame are inherited by its children.

FrameNode_Load is the entrypoint to the Frame Hierarchy Loader, but I'll leave that subject alone until
after a brief discussion of the MeshNode Structure. 
Here is the MeshNode Structure:

MeshNode STRUCT
    LO LinkedObject <>
;	// mesh variables
	pD3DXSkinMesh    LPD3DXSKINMESH ?	;< Pointer to the D3DX skin mesh
	pD3DXBlendedMesh   LPD3DXMESH  ? 	;< Pointer to the blended version
	pAdjacency                   DWORD ?       	;< The adjacency array
;	// materials
	pMeshMaterials            DWORD ?		;< Pointer to the array of material structs
	pMeshTextures             DWORD ?      	;< The array of pTexture dwords
	dwNumMaterials           DWORD ?		;< The number of materials
;	// skinning info
	dwAttrCount                   DWORD  ?                 ;< The attribute count
            dwMaxFaceInfluence    DWORD ?	            ;< Maximum face influence
	pBoneMatrix                 LPD3DXMATRIX  ?	;< Bones' matrices array
	pBoneNamesBuf          LPD3DXBUFFER ?	;< Bones' names buffer
	pBoneOffsetMatrix        LPD3DXMATRIX ?	;< Bones' offset matrices
	pBoneCombinationBuf LPD3DXBUFFER ?     ;< Bone combinations buffer
MeshNode ENDS

It contains LinkedList support and can be divided into three sections:
The Mesh Variables section contains fields which pertain to the D3D Mesh Interface.
The Mesh Materials section contains fields pertaining to the materials and textures of this skinned mesh.
The Skinning Info section contains fields pertaining to the Bones used to animate the skinned mesh.
Bones are a virtual concept made a lot easier when we use a 3D model editor application like Maya.
In Maya, a model's 3D mesh can have a skeleton of bones, and then the skeleton and the mesh
can be fused, so that when you manipulate the bones, the mesh is deformed to match.
Back in the real world, a bone is a transformation applied to a smaller group of vertices of a larger mesh.
That means a bone is nothing but a matrix defining a transformation to be applied to a group of vertices.
A bonecombination is the group of vertices affected by a bone.
The skeleton is a set of these bone matrices, and we keep multiple copies of it called animation frames.
Therefore an animation frame, unlike the frames we've met so far, is a pose for the model,
it's a full set of matrices for transforming all the bones of a skeleton for a particulal pose.
Animations are a series of these animation frames.
An animation can have a name, like "Silly Walk #17", or "FroggyJump".
A model can have several named animations, kept together as an "Animation Set".
The AnimationSet for a skinmesh model is stored under a root pointer in the SkinMesh structure.

That concludes chapter one of the SkinMesh Support Documentation.



========================================================================================== 
Documentation for SkinMesh Support Code
EvilHomer , September 17, 2003
Chapter Two - Discussing the Frames Hierarchy
========================================================================================== 
FrameNode_Load is the entrypoint to the Frame Hierarchy Loader.
We hand the procedure an "xfile object" to examine, and the procedure will
determine what kind of object it is and load it,
and it will also check if the object contains embedded "xfile objects".
If it does, it will call ITSELF with regards to the embedded object(s).

This procedure takes three input parameters: pThis,  pxofobj  and pskinmesh.
The last parameter, pskinmesh, is a constant pointer to the Owner skinmesh object that we are loading.
The second parameter is a pointer to the "xfile object" to be examined.
The first parameter is a pointer to the "current framenode", if we have one.
The return value is E_FAIL for error, otherwise its a pointer to the new FrameNode.

When this procedure is called the very first time, pThis = NULL, and so the procedure will create a FrameNode
to load the frame contents into. 
But when it calls ITSELF, it hands a pointer to an existing frame to associate any new frames with.

The frames in a hierarchy are linked as follows:
Parent-Child links are associated with hierarchy nesting depth, 
while Older-Younger Sibling links associate Sibling frames at a given nesting depth.

The PsuedoCode so far looks something like this:

SkinMesh_Create proc
    open xfile
    for all toplevel objects,
        call FrameNode_Load ,NULL  (the procedure will create a frame and return pointer to it)
    end for
SkinMesh_Create endp

FrameNode_Load proc pCurrentFrame
    determine object type
    if object=frame
        create a new frame container and append it to the current one
        call FrameNode_Load ,CurrentFrame
    elseif object=mesh
        create mesh container
        load mesh into it
    elseif object=animationset
        create animationnode container
        for all subobjects
            call AnimationNode_Load
        end for
    endif
FrameNode_Load endp


