
SkinMesh_LoadAnimationSet PROTO :DWORD,:DWORD
SkinMesh_LoadAnimation      PROTO :DWORD,:DWORD

;=============================================================================
FrameNode STRUCT
	LO LinkedObject <>
	pMeshes LPMESHNODE ?		            ;	< Meshes in this Frame
	OriginalMatrix D3DXMATRIX <>		;	< The matrix originally loaded with the frame
	TransformationMatrix D3DXMATRIX <>	;	< The transformation of the frame
; This matrix is the one that's updated at each rendering operation.
; It's obtained by combining the matrix of this frame with the matrix 
; of the parent frame. */
	CombinedMatrix D3DXMATRIX <>       	;            < The combined matrix
FrameNode ENDS
LPFRAMENODE typedef ptr FrameNode

SkinMesh STRUCT
    pFramesRoot       LPFRAMENODE ?             ;Root of entire frame hierarchy of this SkinMesh Model
    pAnimationsRoot LPANIMATIONNODE ?       ;Root of entire animation set of this SkinMesh Model
SkinMesh ENDS
LPSKINMESH typedef ptr SkinMesh


;=============================================================================
;This procedure parses all the "File Objects" in a given File Object, loading them into a Frame Hierarchy.
;If it finds instances of a "Frame Object" while doing so, it will iteratively call itself.
;Noting that it performs an Append of the new FrameNode onto the existing one,
;what happens if we find more than one "Frame Object" within a Frame?
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
FrameNode_Load PROC  pThis:LPFRAMENODE, pxofobj:DWORD ,  pskinmesh:DWORD
local dwsize, dwnamelength, pmeshnode :DWORD
local objecttype:LPGUID
local pxofchildobj:ptr DIRECTXFILEDATA 
local pxofchild:ptr DIRECTXFILEOBJECT
local pxofdata:ptr
local pmat:LPD3DXMATRIX 
local pchildframe:LPFRAMENODE
local TheName[256]:BYTE
local ErrBuf[256]:BYTE


mov esi,pThis
assume esi:LPFRAMENODE

pushad
invoke MessageBox,0,[esi].LO.pName,CTXT("Frame being loaded"),MB_OK
popad

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
; get the type of the object
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
.if ($mcall (pxofobj,IDirectXFileData_GetType,addr objecttype))!= D3D_OK
	  Errr CTXT ( "Couldn't get x file data object's type")
.endif       
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
;Let's see what kind of object we've been handed to process...
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
invoke IdentifyGUID, objecttype
.if eax==Unknown
    Errr CTXT("Failed to identify xfile object !!")
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

.elseif eax==Mesh
    invoke MessageBox,0,CTXT("MESH !!"),CTXT("Frames Loader"),MB_OK

;** Create a meshnode, append it to this frame's list of meshes, and load a mesh into it
;** Note the usage of NULL for target object indicates it needs to be created as well as linked
                        mov pmeshnode,$invoke (LinkedLists_Append, [esi].pMeshes, NULL, sizeof MeshNode)
                        .if [esi].pMeshes==NULL                             ;is the list's root object NULL meaning no existing entries?
                                m2m [esi].pMeshes,pmeshnode         ;if it is, then this meshnode is the new Root (top of List)
                        .endif
                        invoke MeshNode_Create, pmeshnode, pxofobj  ;ok now attempt to load the mesh
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    .elseif eax==FrameXFormMatrix  
;		// fetch pointer to copy of the matrix data
;                       // then copy the data from the matrix into our frame's Transform and Original matrices
		mcall [pxofobj],IDirectXFileData_GetData, NULL, addr dwsize, addr pmat 
                        .if eax==DXFILE_OK
                              push esi
                              push edi
                              mov esi,pmat
                              lea edi,[esi].TransformationMatrix 
                              lea ebx,[esi].OriginalMatrix
                              mov ecx,dwsize
                              .while ecx
                                    mov al,byte ptr[esi]
                                    mov byte ptr[edi],al
                                    mov byte ptr[ebx],al
                                    inc esi
                                    inc edi
                                    inc ebx
                                    dec ecx 
                              .endw                                     
                              pop edi
                              pop esi
  		.else
  		      Errr CTXT ("Couldn't read frame transformation matrix from an x file")
  		.endif

;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    .elseif eax==AnimationSet 
        invoke MessageBox,0,CTXT("AnimationSet !!"),CTXT("Frames Loader"),MB_OK
  ;      invoke SkinMesh_LoadAnimationSet, pskinmesh, pxofobj 
;------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    .elseif eax==Animation
    invoke MessageBox,0,CTXT("Animation !!"),CTXT("Frames Loader"),MB_OK
   ;     invoke SkinMesh_LoadAnimation,  pskinmesh, pxofobj
;-------------------------------------------------------------------------------------------------------------------------------------------------------

    .elseif eax==Frame
    invoke MessageBox,0,CTXT("Frame !!"),CTXT("Frames Loader"),MB_OK

;-------------------------------------------------------------------------------------------------------------------------------------------------------
;                   first we'll create a new child frame container structure and link it to the current one
                    mov pchildframe,$invoke (LinkedLists_AppendSibling, esi, NULL, sizeof FrameNode)
;-------------------------------------------------------------------------------------------------------------------------------------------------------
;                  now we get & set the name of the child frame
                      mov dwnamelength,   0
            	mcall [pxofobj],IDirectXFileData_GetName, NULL, addr dwnamelength 
		.if eax!=D3D_OK
                                    pop esi
			Errr CTXT( "Couldn't get the frame name length")
		.endif		
		.if dwnamelength > 0 		
			mcall [pxofobj],IDirectXFileData_GetName, addr TheName, addr dwnamelength 
			.if TheName[0]==0
		              	invoke NewName,pchildframe,CTXT("(Nameless)")
			.else
                                        pushad
    	       		    invoke NewName,pchildframe,addr TheName		
                                        .if pThis!=NULL
                                            mov esi,pThis
                                            invoke wsprintf,addr ErrBuf,CTXT("New Frame = %s",13,10,"Parent Frame=%s"),addr TheName,[esi].LO.pName
                                        .else
                                            invoke wsprintf,addr ErrBuf,CTXT("New Frame = %s",13,10,"Parent Frame=(NULL)"),addr TheName
                                        .endif
                                        invoke MessageBox,0,addr ErrBuf,CTXT("Sub Frame"),MB_OK
                                        popad
                                    .endif
                        .else
                                    invoke MessageBox,0,addr ErrBuf,CTXT("Nameless frame?"),MB_OK

		.endif

;-------------------------------------------------------------------------------------------------------------------------------------------------------
;         Enumerate child objects of a Frame - we'll need to ITERATE, and do it from within a Loop
;        because we don't know how many File Objects we'll find in the Frame.
;        - Child object can be data, data reference or binary.
;        - Use QueryInterface() to find what objecttype of object a child is.
;-------------------------------------------------------------------------------------------------------------------------------------------------------
                            ;Scan object for embedded objects
;-------------------------------------------------------------------------------------------------------------------------------------------------------
    mcall [pxofobj], IDirectXFileData_GetNextObject, ADDR pxofchild 
    .while eax == DXFILE_OK
        invoke MessageBox,0,addr TheName,CTXT("Frame contains embedded objects"),MB_OK

;#1- Scan object for embedded REFERENCE objects
        mcall [pxofchild], IDirectXFileObject_QueryInterface, ADDR IID_IDirectXFileDataReference, ADDR pxofdata 
        .if eax==DXFILE_OK	                                                         ;handle a potentially nested situation
              mcall [pxofdata], IDirectXFileDataReference_Resolve, addr pxofchildobj
               .if eax==DXFILE_OK	
                    invoke MessageBox,0,addr TheName,CTXT("Parsing embedded reference objects"),MB_OK
                    invoke FrameNode_Load, pchildframe, pxofchildobj, pskinmesh   ;by performing a nested call on myself
                    _saferelease pxofchildobj
               .endif
               _saferelease pxofdata
        .endif

;#2- Scan object for embedded NONREFERENCE objects
        mcall [pxofchild], IDirectXFileObject_QueryInterface, ADDR IID_IDirectXFileData, ADDR pxofchildobj
        .if eax == DXFILE_OK
              invoke MessageBox,0,addr TheName,CTXT("Parsing embedded objects"),MB_OK
	  invoke FrameNode_Load,  pchildframe, pxofchildobj, pskinmesh   ;by performing a nested call on myself
             .if eax==S_OK
                     invoke MessageBox,0,CTXT("Current Frame Parsed",13,10,"Looping for siblings"),CTXT("COOLNESS"),MB_OK
             .endif
     	 _saferelease pxofchildobj
        .endif
        _saferelease pxofchild	
        assume esi:nothing
        mcall [pxofobj], IDirectXFileData_GetNextObject, ADDR pxofchild 
    .endw
    pop esi
    invoke MessageBox,0,CTXT("Current Frame Parsed",13,10,"RETURNING"),CTXT("COOLNESS"),MB_OK
;=============================================================================
.else
        Errr CTXT ("UNHANDLED OBJECT TYPE!")
.endif


@@:
return S_OK
FrameNode_Load ENDP

;=============================================================================
;=============================================================================
;=============================================================================
FrameNode_Destroy PROC pThis:LPFRAMENODE
.if pThis==NULL
    Errr CTXT ("MORON Tried to Destroy NULL FrameNode")
.endif
mov esi,pThis
assume esi:LPFRAMENODE
.if [esi].LO.pChild !=NULL
    invoke FrameNode_Destroy,[esi].LO.pChild
.endif
invoke MeshNode_Destroy, [esi].pMeshes
pop esi
ret
FrameNode_Destroy ENDP
;=============================================================================
;=============================================================================
;=============================================================================
; Compute the world matrix of the frame by combining 
;the transformation matrix of this frame with the parent's matrix
; pmat : A pointer to the parent's world matrix

FrameNode_Update PROC pThis:LPFRAMENODE, pmat:LPD3DXMATRIX 
    .if pThis==NULL 
        ret
    .endif
    push esi
    mov esi,pThis
    assume esi:LPFRAMENODE
    invoke D3DXMatrixMultiply, addr [esi].CombinedMatrix, addr [esi].TransformationMatrix, pmat 
;// update child frames
    .if [esi].LO.pChild !=0
          mov esi,[esi].LO.pChild
          .while esi!=0
  	    invoke FrameNode_Update, [esi].LO.pChild, addr [esi].CombinedMatrix
	    mov esi,[esi].LO.pChild
          .endw
    .endif
    pop esi
ret
FrameNode_Update ENDP
;=============================================================================
;=============================================================================
;=============================================================================


FrameNode_Render PROC pThis:LPFRAMENODE
            assume esi:nothing
            .if pThis==NULL                                                 ;Theres only one input param, and its REQUIRED !!
                ret
            .endif
            push esi
            mov esi,pThis                                   ;If we have child meshes, set the world matrix and rend them
            assume esi:LPFRAMENODE
	.if [esi].pMeshes != NULL
                push esi
      	    mcall [lpd3dDevice],IDirect3DDevice8_SetTransform, 256, addr [esi].CombinedMatrix 
                assume esi:LPFRAMENODE
    	    mov esi,[esi].pMeshes       	;Fetch RootPointer of this Frame's MeshNodes linkedlist
                assume esi:LPMESHNODE
    	    .while esi!=NULL	
    		invoke MeshNode_Render, esi       ;Render any and all the MeshNodes for this FrameNode
    		mov esi,[esi].LO.pChild
    	    .endw
                pop esi 
            .endif

           .if [esi].LO.pChild!=NULL                    ;If we have child frames, iteratively rend them
                  mov esi,[esi].LO.pChild
      	     .while esi!=0	
      	         invoke FrameNode_Render,esi
      	         mov esi,[esi].LO.pChild
                 .endw
	.endif
            pop esi
            return S_OK
FrameNode_Render ENDP
;=============================================================================
;=============================================================================
;=============================================================================
; Find a frame in this frame hierarchy with the specified name.

FrameNode_FindFrame PROC pThis:DWORD, pName:DWORD
        .if pThis==NULL || pName==NULL 
            Errr CTXT ("FrameNode_FindFrame requires BOTH params, neither can be NULL !!")
        .endif
        push esi
        mov esi,pThis
        assume esi:LPFRAMENODE
        mov eax,[esi].LO.pName
        .if pName == eax                                                    ;Quickie test - Are the stringpointers an exact match?
	return pThis
        .else
            invoke lstrcmp,pName,[esi].LO.pName             ;Generic test - Are the two actual strings an exact match?
            .if eax==0
                return pThis
            .endif
        .endif
;		// loop through the child objects
	.while [esi].LO.pChild != NULL		
	     invoke FrameNode_FindFrame ,[esi].LO.pChild, pName
	      .break .if eax!=NULL
	      mov esi,[esi].LO.pChild
	.endw
            pop esi
	ret
FrameNode_FindFrame ENDP
;=============================================================================
;=============================================================================
;=============================================================================
; Find the bones of the child meshes of a frame.
; prootframe = The root of the frame hierarchy to search in for the bones.

FrameNode_FindBones PROC pThis:LPFRAMENODE, prootframe:LPFRAMENODE
.if prootframe==NULL
    Errr CTXT ("MORON Tried to Find Bones of a NULL FrameNode")
.endif
push esi
;	// find the bones of the meshes
            mov ebx,pThis
            push ebx
            assume ebx:LPFRAMENODE
            mov esi, [ebx].pMeshes
            assume esi:LPMESHNODE            
            .while esi!=NULL
		invoke MeshNode_FindBones, esi ,prootframe
		mov esi,[esi].LO.pChild
	.endw
            pop esi
;	// find the bones in the child frames
            assume esi:LPFRAMENODE
	mov esi,[esi].LO.pChild
	.while esi!=NULL	
		invoke FrameNode_FindBones,esi, prootframe
		mov esi,[esi].LO.pChild
	.endw
pop esi
ret
FrameNode_FindBones ENDP
;=============================================================================
;=============================================================================
;=============================================================================
;=============================================================================
;This is just a helper to set the TransformationMatrix for a given Frame
;=============================================================================
FrameNode_SetTransformationMatrix PROC pFrame:DWORD, pmat:DWORD
.if pFrame==NULL 
    Errr CTXT("MORON Tried to Set Transformation Matrix on a NULL Frame")
.endif
push esi
mov esi,pFrame
assume esi:LPFRAMENODE
m2m [esi].TransformationMatrix, pmat
pop esi
ret
FrameNode_SetTransformationMatrix ENDP


;=============================================================================
FrameNode_InitObject PROC
invoke D3DXMatrixIdentity, addr [esi].TransformationMatrix 
invoke D3DXMatrixIdentity, addr [esi].OriginalMatrix 
ret
FrameNode_InitObject ENDP
