;=====================================================================
;=====================================================================
;CONTENTS OF THIS INCLUDE FILE
;=====================================================================
;SkinMesh_Create
;SkinMesh_Destroy
;SkinMesh_LoadAnimation
;SkinMesh_LoadAnimationSet
SkinMesh_FindBones PROTO pRoot:DWORD
;SkinMesh_Render

SkinMesh STRUCT
    pFramesRoot       LPFRAMENODE ?             ;Root of entire frame hierarchy of this SkinMesh Model
    pAnimationsRoot LPANIMATIONNODE ?       ;Root of entire animation set of this SkinMesh Model
SkinMesh ENDS
LPSKINMESH typedef ptr SkinMesh

;=====================================================================
;=====================================================================
;=====================================================================
SkinMesh_Create PROC  filename:DWORD
local pThisSkinMesh:DWORD
local pframe:DWORD
local pxofapi:DWORD
local pxofenum:DWORD
local pxofobj:DWORD


;=====================================================================
;Create a "DirectXFile" interface object (so we can call functions in the interface), 
;Register the Retained Mode Templates (so that we can recognize them),
;and Create an "Enumeration Object" (so that we can enumerate "top level objects")
;=====================================================================

.if $invoke (DirectXFileCreate, addr pxofapi) != DXFILE_OK
    Errr CTXT ("Couldn't create IDirectXFile")
.endif

mcall [pxofapi],IDirectXFile_RegisterTemplates,addr D3DRM_XTEMPLATES,D3DRM_XTEMPLATE_BYTES
.if eax!=DXFILE_OK
     _saferelease pxofapi
     Errr CTXT ( "Couldn't register D3DRM templates")
.endif

mcall [pxofapi],IDirectXFile_CreateEnumObject,filename, DXFILELOAD_FROMFILE, addr pxofenum
.if eax!=DXFILE_OK
     _saferelease pxofapi
    Errr CTXT ( "Couldn't create x file enum object")
.endif
;=====================================================================
;     Create a SkinMesh container object to hold the goodies
;=====================================================================

mov pThisSkinMesh,$invoke (AppendOrInsert,NULL,sizeof SkinMesh) ;Create SkinMesh
mov pframe,$invoke (AppendOrInsert,NULL,sizeof FrameNode)            ;Create RootFrame
invoke NewName, pframe, CTXT("$ROOT$")
.if eax==NULL
    Errr CTXT ("Failed to create a named Frame Container Object - out of memory?")
.else
    mov esi,pThisSkinMesh                       ;Attach RootFrame to SkinMesh
    assume esi:ptr SkinMesh
    mov [esi].pFramesRoot,eax
    mov [esi].pAnimationsRoot,eax
;=====================================================================
;     Enumerate toplevel objects of the xfile
;=====================================================================
    mcall [pxofenum], IDirectXFileEnumObject_GetNextDataObject, ADDR pxofobj     
    .while eax == DXFILE_OK
         invoke FrameNode_Load, pxofobj, pframe,pThisSkinMesh
        _saferelease pxofobj
        mcall [pxofenum], IDirectXFileEnumObject_GetNextDataObject, ADDR pxofobj     
    .endw
    invoke MessageBox,0,CTXT("All TopLevel Objects Loaded",13,10,"Beginning Processing"),CTXT("COOLNESS"),MB_OK

      
;=====================================================================
; Having loaded a complete frame hierarchy, we should Bone It Out lol
;=====================================================================
	invoke SkinMesh_FindBones , pThisSkinMesh     ;Find the bones of this hierarchy
	_saferelease pxofapi 
	_saferelease pxofenum 
             return pThisSkinMesh
.endif
SkinMesh_Create ENDP
;=====================================================================
;=====================================================================
;=====================================================================

SkinMesh_Destroy PROC pSkinny:ptr SkinMesh        
            push esi
            mov esi,pSkinny
            assume esi:ptr SkinMesh
	invoke FrameNode_Destroy, [esi].pFramesRoot
	invoke AnimationNode_Destroy, [esi].pAnimationsRoot
            pop esi
            ret
SkinMesh_Destroy ENDP

;=====================================================================
;=====================================================================
;=====================================================================
; Load an animation from an x file data object
;pxofobj = The x file data object to load from
;pparent = The parent node on which to attach the new created animation node.
;		Specify NULL to add the animation at the root level (if the animation is not
;		part of an animation set).
;=====================================================================
;Begin
;  Create new CAnimationNode object
;  Attach the new object to the link list
;  For every child template
;    Call CAnimationNode::Load for the new animation object
;End
;=====================================================================
SkinMesh_LoadAnimation PROC  pxofobj:DWORD , pSkinMesh:DWORD
;=====================================================================
local panimation:ptr CAnimationNode 
local pxofchildobj:ptr 
;=====================================================================
mov esi,pSkinMesh
assume esi:LPSKINMESH
;	// create the new object
            mov panimation, $invoke (LinkedLists_Append, [esi].pAnimationsRoot, NULL, sizeof AnimationNode)
	.if pSkinMesh== NULL 	; if no parent node is specified, remember the head of the list
                    m2m [esi].pAnimationsRoot, panimation
            .endif

;	// load the animation
@@:    mcall [pxofobj],IDirectXFileData_GetNextObject, addr pxofchildobj 
            .if eax!=SUCCEEDED
                jmp @F
            .endif            	
	invoke AnimationNode_Load, panimation, pxofchildobj
            _saferelease pxofchildobj
	jmp @B
@@:
            ret
SkinMesh_LoadAnimation ENDP
;=====================================================================
;=====================================================================
;=====================================================================
;This proc loads an animation-set from an xfile data object and attaches it to a Frame
;pParent = The frame node in which the AnimationSet will be stored
;pxofobj : The x file data object to load from.

SkinMesh_LoadAnimationSet PROC pOwnerSkinMesh:DWORD, pxofobj:DWORD 
local panimationset:LPFRAMENODE
local szname[256]:BYTE
local dwnamelength:DWORD 
local pxofchildobj:PTR 
local pxofchilddataobj:PTR 
local Rtype:LPGUID
.if pOwnerSkinMesh==NULL
    Errr CTXT("MORON Tried to Load AnimationSet into NULL SkinMesh")
.endif



; Create the animation-node and attach it to the Parent Frame
mov esi,pOwnerSkinMesh
assume esi:LPSKINMESH
mov panimationset,$invoke (LinkedLists_Append, pOwnerSkinMesh, [esi].pAnimationsRoot, sizeof AnimationNode)
.if [esi].pAnimationsRoot==NULL
    m2m  [esi].pAnimationsRoot, panimationset
.endif

;Get and Set the name of the animation set


mcall [pxofobj],IDirectXFileData_GetName, NULL, addr dwnamelength 
.if eax!=D3D_OK
	Errr CTXT ( "Couldn't get animation set name length")
.endif
.if dwnamelength > 0 
	mcall [pxofobj],IDirectXFileData_GetName, addr szname, addr dwnamelength 
	.if eax== FAILED
		Errr CTXT ( "Couldn't get animation set name")
	.endif
	invoke NewName, panimationset, addr szname 

	;Query through the child objects to load the animations (parsing)
@@:
	mcall [pxofobj],IDirectXFileData_GetNextObject, addr pxofchildobj
            .if eax!=SUCCEEDED
                jmp @F
            .endif
; Query the child for it's FileData
	mcall [pxofchildobj],IDirectXFileObject_QueryInterface,addr IID_IDirectXFileData, addr pxofchilddataobj
	.if eax== SUCCEEDED
		mcall [pxofchilddataobj],IDirectXFileData_GetType,addr  Rtype 
		.if eax==FAILED
			_saferelease pxofchildobj
			_saferelease pxofchilddataobj
			Errr CTXT ( "Couldn't get object type while loading animation set" )
		.endif    
                      .if $invoke (CmpGUID, Rtype, ADDR  TID_D3DRMAnimation,16) == TRUE
			invoke SkinMesh_LoadAnimation, pxofchilddataobj, panimationset 
                       .endif
	           _saferelease pxofchilddataobj
 	 .endif
	_saferelease pxofchildobj
            jmp @B
.endif
@@:
ret
SkinMesh_LoadAnimationSet ENDP
;=====================================================================
;=====================================================================
;=====================================================================
SkinMesh_FindBones PROC pSkinny:DWORD
            push esi
            mov esi,pSkinny
            assume esi:ptr SkinMesh
	invoke FrameNode_FindBones, [esi].pFramesRoot, [esi].pFramesRoot
	invoke AnimationNode_FindBone, [esi].pAnimationsRoot, [esi].pFramesRoot
            pop esi
            ret
SkinMesh_FindBones ENDP
;=====================================================================
;=====================================================================
;=====================================================================
SkinMesh_Render PROC  pThis:ptr SkinMesh, pworldmat:LPD3DXMATRIX 
            push esi
            mov esi,pThis
            assume esi:ptr SkinMesh
	invoke FrameNode_Update,[esi].pFramesRoot, pworldmat          ; calculate world matrix of all bones
	invoke FrameNode_Render,[esi].pFramesRoot                    	; render the meshes
            pop esi
            ret
SkinMesh_Render ENDP
;=====================================================================
;=====================================================================
;=====================================================================